describe('Rev.Store', function() {
  var store;
  var mock = {forceUpdate: function() {}, props: {}};
  var dataWithLinks = {
      "data": [
        {
          "id": "25",
          "type": "contacts",
          "links": {
            "user": {
              "linkage": {
                "type": "users",
                "id": "1"
              }
            }
          }
        }
      ],
      "included": [
        {
          "id": "3",
          "name": "Best team",
          "type": "orgs"
        },
        {
          "id": "1",
          "type": "users",
          "links": {
            "org": {
              "linkage": {
                "type": "orgs",
                "id": "3"
              }
            }
          }
        }
      ]
  };

  Rev.appModel('Contact', BaseModel.forResource('contacts'));
  Rev.appModel('Contact.Collection', BaseCollection.forResource('contacts'));
  Rev.appModel('User', BaseModel.forResource('users'));
  Rev.appModel('User.Collection', BaseCollection.forResource('users'));
  Rev.appModel('Org', BaseModel.forResource('orgs'));
  Rev.appModel('Org.Collection', BaseCollection.forResource('orgs'));

  beforeEach(function() {
    store = new Rev.Store(mock);
  });

  it('can attach to a component and cause it to re-render', function() {
    spyOn(store, '__changeCallback');
    store.triggerChange();
    expect(store.__changeCallback).toHaveBeenCalled();
  });

  it('can detach from a component', function() {
    store.detach();
    expect(store.__changeCallback).toEqual(null);
  });

  it('can load collections from a list', function() {
    var list = [
      {type: 'users', id: '1'},
      {type: 'orgs', id: '1'},
      {type: 'users', id: '2'}
    ];
    store.loadCollectionsFrom(list);

    expect(store.users.length).toEqual(2);
    expect(store.orgs.length).toEqual(1);
  });

  it('can make a new model attached to the store', function() {
    store.new('foo', BaseModel, {type: 'users', id: '1'});

    expect(store.foo.get('type')).toEqual('users');
    expect(store.foo.get('id')).toEqual('1');
  });

  it('can get an attribute of a model by JSON path', function() {
    store.new('foo', BaseModel, {type: 'users', id: '1'});

    expect(store.get('foo.type')).toEqual('users')
  });

  it('can set an attribute of a model by JSON path', function() {
    store.new('foo', BaseModel, {type: 'foos', id: '1'});
    store.set('foo.bar', 'success');

    expect(store.get('foo.bar')).toEqual('success');
  });

  it('can follow links to included objects (with 0 links)', function() {
    store.loadCollectionsFrom(dataWithLinks.data);
    store.loadCollectionsFrom(dataWithLinks.included);

    expect(
      store.getLink(
        'contacts.models[0]'
      ).get('type')).toEqual('contacts');
  });

  it('can follow links to included objects (with 1 links)', function() {
    store.loadCollectionsFrom(dataWithLinks.data);
    store.loadCollectionsFrom(dataWithLinks.included);

    expect(
      store.getLink(
        'contacts.models[0]',
        'user'
      ).get('type')).toEqual('users');
  });

  it('can follow links to included objects (with multiple links)', function() {
    store.loadCollectionsFrom(dataWithLinks.data);
    store.loadCollectionsFrom(dataWithLinks.included);

    expect(
      store.getLink(
        'contacts.models[0]',
        'user',
        'org'
      ).get('type')).toEqual('orgs');
  });
})
